"""
@Name: Motif
@Author: Jun Ding
@Data: Apr.7,2011
@Version:1.2

function: These are modules for motif object.
Module 1: MotifScan1
This module is used to scan the sequence and then get the motif hits at a given cutoff.
usage: MotifScan(Motif,Sequence,cutoff)
output: Motif hit position list.
Motif Representation type:
Position weight Matrix

Module2: MotifScan2
This module is used to scan sequence and then get the motif hits at a given cutoff
usage: MotifScan(Motif,Sequence,cutoff)
output: Motif hits position list.
Motif Representation type:
Consensus

Module3:rmRedMotifHit
This module is used to remove redundant motif hits.
usage: MotifScan(motif_list,hit_motif_list,motif_hit_poistion_list,cutoff)
cutoff: The minimal distance between two hits,which will be regarded as two hits.
 If smaller than the cutoff,then two hits well be
regarded as one hits, hits with smaller score will be removed.

Module4: MotifScore
This module is used to get the score of given motif
usage: MotifScore(motif)


*Note: The input sequence must be all upper case.

__main__ usage:
python Motif.py -m 1 <line_format_seq_file> <matrix_motif_file score_cutoff>
python Motif.py -m 2 <line_format_seq-file> <line_format_consensus_motif_file> <percentage_cutoff>
python motif.py -m 4 <line_format_matrix_motif_file> <line_format_motif_hits_file> <line_format_motif_positions_file> <minimal_distance_cutoff>(normally set as 4)


"""
import math

#module MotifScan1, use motif matrix to scan sequence
def MotifScan1(seq,mot,cut):
	hit_position=[]
	for im in range(len(mot)):
		mot[im]=[float(item) for item in mot[im]]
	bk=[0,0,0,0]
	bk[0]=0.290763621993511
	bk[1]=0.202156441211073
	bk[2]=0.205656409776978
	bk[3]=0.301423527018438
	if len(seq)-len(mot)>=0:
		for i in range(len(seq)-len(mot)+1):
			score_i=0
			for j in range(len(mot)):
				if seq[i+j]=='A':
					score_i=score_i+math.log(mot[j][0]/(bk[0]))
				elif seq[i+j]=='C':
					score_i=score_i+math.log(mot[j][1]/(bk[1]))
				elif seq[i+j]=='G':
					score_i=score_i+math.log(mot[j][2]/(bk[2]))
				elif seq[i+j]=='T':
					score_i=score_i+math.log(mot[j][3]/(bk[3]))
				else:
					score_i=score_i+0
			if score_i>cut:
				hit_position.append(str(i))
	else:
		print('error! sequences is shorter than motif')
	return hit_position

#module MotifScan2, use motif consensus to scan sequence
def MotifScan2(s,m,p):
	## m:a motif seq
	## s:a gene seq
	flag_hit=0
	hit_position=[]
	n_n=0
	for t_m in range(len(m)):
		if (m[t_m]=='A')or(m[t_m]=='C')or(m[t_m]=='G')or(m[t_m]=='T'):
			n_n=n_n+1
	for t in range(len(s)-len(m)+1):
		ct=0
		d_mismatch=0
		for k in range(len(m)):
			if (s[t+k]==m[k])&(s[t+k]!='N'):
				ct=ct+1
			else:
				f1=(s[t+k]=='A')
				f2=(s[t+k]=='C')
				f3=(s[t+k]=='G')
				f4=(s[t+k]=='T')
				if m[k]=='R':
					if not f1|f3:
					    d_mismatch=1
					    break
				elif m[k]=='Y':
					if not f2|f4:
					    d_mismatch=1
					    break
				elif m[k]=='K':
					if not f3|f4:
					    d_mismatch=1
					    break
				elif m[k]=='M':
					if not f1|f2:
					    d_mismatch=1
					    break
				elif m[k]=='S':
					if not f2|f3:
					    d_mismatch=1
					    break
				elif m[k]=='W':
					if not f1|f4:
					    d_mismatch=1
					    break
				elif m[k]=='B':
					if not f2|f3|f4:
					    d_mismatch=1
					    break
				elif m[k]=='D':
					if not f1|f3|f4:
					    d_mismatch=1
					    break
				elif m[k]=='H':
					if not f1|f2|f4:
					    d_mismatch=1
					    break
				elif m[k]=='V':
					if not f1|f2|f3:
					    d_mismatch=1
					    break
		if (float(ct)/n_n>=p)and(d_mismatch==0):
			hit_position.append(str(t))
	return hit_position

def motif_score(input_motif):
	score=len(input_motif)
	return score

def rmRedMotifHit(motifs,Hits,Positions,cutoff):
	for im in range(len(Positions)-1):
		for jm in range(im+1,len(Positions)):
			if (Positions[im]!=[])and(Positions[jm]!=[]):
				if abs(int(Positions[im])-int(Positions[jm]))<cutoff:
					scorei=motif_score(motifs[int(Hits[im])])
					scorej=motif_score(motifs[int(Hits[jm])])
					if scorej<=scorei:
						Positions[jm]=[]
						Hits[jm]=[]
	Hits=[item for item in Hits if item!=[]]
	Positions=[item for item in Positions if item!=[]]
	return Hits,Positions

if __name__=="__main__":
	###########################################
	import sys,getopt,pdb
	optlist,args=getopt.getopt(sys.argv[1:],'m:')
	################read in the seq file
	if len(optlist)<1:
		print('usage: python Motif.py -m module: 1 or 2 line-format-Seq-file motif-file')
	else:
		mode=optlist[0][1]
		if mode=="1":
			f=open(args[0],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			seqs=lf
			f=open(args[1],'r')
			lf=f.readlines()
			f.close()
			lf=''.join(lf)
			lf=lf.split('>')[1:]
			lf=[item.split('\n')[:-2] for item in lf]
			motifs=[]
			cutoff=[]
			for i in lf:
				t=i[1:]
				t=[item.split(' ') for item in t]
				motifs.append(t)
				cut=float(i[0].split('\t')[1])
				cutoff.append(cut)
			#pdb.set_trace()
			HitPosition=[]
			HitMotif=[]
			for i in range(len(seqs)):
				HitMotif_i=[]
				HitPosition_i=[]
				for j in range(len(motifs)):
					position=MotifScan1(seqs[i],motifs[j],cutoff[j])
					if position!=[]:
						HitPosition_i=HitPosition_i+position
						HitMotif_i=HitMotif_i+[str(j)]*len(position)	
				print(i)
				HitPosition.append(HitPosition_i)
				HitMotif.append(HitMotif_i)
			HitMotif=[' '.join(item) for item in HitMotif]
			HitMotif='\n'.join(HitMotif)
			HitPosition=[' '.join(item) for item in HitPosition]
			HitPosition='\n'.join(HitPosition)
			f=open(args[0]+"_MotifHits",'w')
			f.write(HitMotif)
			f.close()
			f=open(args[0]+'_MotifHitsPositions','w')
			f.write(HitPosition)
			f.close()
		if mode=='2':
			f=open(args[0],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			seqs=lf
			f=open(args[1],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			motifs=lf
			cut=float(args[2])
			HitPosition=[]
			HitMotif=[]
			for i in range(len(seqs)):
				HitMotif_i=[]
				HitPosition_i=[]
				for j in range(len(motifs)):
					position=MotifScan2(seqs[i],motifs[j],float(cut))
					if position!=[]:
						HitPosition_i=HitPosition_i+position
						HitMotif_i=HitMotif_i+[str(j)]*len(position)	
				HitPosition.append(HitPosition_i)
				HitMotif.append(HitMotif_i)
				print(i)
			HitMotif=[' '.join(item) for item in HitMotif]
			HitMotif='\n'.join(HitMotif)
			HitPosition=[' '.join(item) for item in HitPosition]
			HitPosition='\n'.join(HitPosition)
			args[1]=args[1].split('\t')[-1]
			f=open(args[0]+'_'+args[1]+'_MotifHits','w')
			f.write(HitMotif)
			f.close()
			f=open(args[0]+'_'+args[1]+'_MotifHitsPositions','w')
			f.write(HitPosition)
			f.close()
		if mode=='4':
			f=open(args[0],'r')
			lf=f.readlines()
			f.close()
			lf=''.join(lf)
			lf=lf.split('>')[1:]
			lf=[item.split('\n')[:-2] for item in lf]
			motifs=[]
			for i in lf:
				t=i[1:]
				t=[item.split(' ') for item in t]
				motifs.append(t)
			f=open(args[1],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			lf=[item.split(' ') for item in lf]
			HitFile=lf
			f=open(args[2],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			lf=[item.split(' ') for item in lf]
			PositionFile=lf
			cutoff=int(args[3])
			non_red_Hits=[]
			non_red_Positions=[]
			for i in range(len(PositionFile)):
				Hits_i=HitFile[i]
				Positions_i=PositionFile[i]
				Hits_i,Positions_i=rmRedMotifHit(motifs,Hits_i,Positions_i,cutoff)	
				non_red_Hits.append(Hits_i)
				non_red_Positions.append(Positions_i)
			non_red_Hits=[' '.join(item) for item in non_red_Hits]
			non_red_Positions=[' '.join(item) for item in non_red_Positions]
			non_red_Hits='\n'.join(non_red_Hits)
			non_red_Positions='\n'.join(non_red_Positions)
			f=open(args[1]+"_"+str(cutoff)+'_non_redundant','w')
			f.write(non_red_Hits)
			f.close()
			f=open(args[2]+"_"+str(cutoff)+'_non_redundant','w')
			f.write(non_red_Positions)
			f.close()
			
			


			


			
				
	

