"""
Author: Jun Ding
function: These modules are used for sequences processing.
Input: The input format is the string list. Each string represents a sequence.
Modules:
1. Random Permutation---randomPermutation(seq)
2. Reverse Complement---ReverseCompliment(seq)
3. Convert Fasta format to line format---Fasta2Line(seq)
4. Get the frequency of Nucleotide(TACG)---FreqNucleotie(seq)

"__main__" usage:
python SEQProcess.py -m (1,2,3,4) <seq_file>
"""
#import numpy
#from numpy import *

#Module 1 : Random Permutation
def randomPermutation(SeqString):
	SeqString=[item for item in SeqString]
	p_seq=random.permutation(SeqString)
	p_seq=''.join(p_seq)
	return p_seq

#Module 2: Reverse Complement
def ReverseCompliment(SeqString):
	SeqString=[item for item in SeqString]
	SeqString.reverse()
	d={'A':'T','C':'G','G':'C','T':'A','N':'N','a':'t','c':'g','g':'c','t':'a','n':'n'}
	for i in range(len(SeqString)):
		SeqString[i]=d[SeqString[i]]
	SeqString=''.join(SeqString)
	return SeqString
#Module3: Convert Fasta Format to Line Format
def Fasta2Line(lf):
	l=[]
	for i in range(len(lf)):
		if lf[i][0]=='>':
			l.append(lf[i])
			lf[i]='>'
	lf=''.join(lf)
	lf=lf.split('>')
	lf=[item for item in lf if item!='']
	lf='\n'.join(lf)
	l='\n'.join(l)
	return l,lf
#Module4: calculate the frequency of nucleotide in seq
def FreqNucleotide(seq):
	sum_a=0
	sum_c=0
	sum_g=0
	sum_t=0
	for i in range(len(seq)):		
		if (seq[i]=='A')or(seq[i]=='a'):
			sum_a+=1
		elif (seq[i]=='C')or(seq[i]=='c'):
			sum_c+=1
		elif (seq[i]=='G')or(seq[i]=='g'):
			sum_g+=1
		elif (seq[i]=='T')or(seq[i]=='t'):
			sum_t+=1
	if sum_a+sum_c+sum_g+sum_t>0:
		freq_a=float(sum_a)/(sum_a+sum_c+sum_g+sum_t)
		freq_c=float(sum_c)/(sum_a+sum_c+sum_g+sum_t)
		freq_g=float(sum_g)/(sum_a+sum_c+sum_g+sum_t)
		freq_t=float(sum_t)/(sum_a+sum_c+sum_g+sum_t)
		return freq_a,freq_c,freq_g,freq_t
	else:
		return [0,0,0,0]

if __name__=="__main__":
	import sys,getopt,pdb
	optlist,args=getopt.getopt(sys.argv[1:],'m:')
	if len(optlist)<1:
		print('please select module')
	else:
		mode=optlist[0][1]
		if mode=='3':
		#read in the Fasta Format Sequences
			f=open(args[0],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			l,lf=Fasta2Line(lf)
			f=open(args[0]+'_LineFormat','w')
			f.write(lf)
			f.close()
			g=open(args[0]+'_GeneName','w')
			g.write(l)
			g.close()
		if mode=='4':
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			A=[0,0,0,0]
			for i in range(len(lf)):
				tmp=FreqNucleotide(lf[i])
				tmp=[item*len(lf[i]) for item in tmp]
				A=[A[j]+tmp[j] for j in range(len(A))]
			freqa=float(A[0])/sum(A)
			freqc=float(A[1])/sum(A)
			freqg=float(A[2])/sum(A)
			freqt=float(A[3])/sum(A)
			output='freq_A:'+str(freqa)+'\n'+'freq_C:'+str(freqc)+'\n'+'freq_G:'+str(freqg)+'\n'+'freq_T:'+str(freqt)
			f=open('frequency_nucleotide_'+args[0],'w')
			f.write(output)
				
		if mode=='1':
			f=open(args[0],'r')
			lf=f.readlines()
			f.close()
			lf=[item.strip() for item in lf]
			lf=[randomPermutation(item) for item in lf]
			lf='\n'.join(lf)
			f=open('randomPermuted_'+args[0],'w')
			f.write(lf)
			f.close()
				
				
		
		
	
