package mpredict;

import java.io.IOException;

import java.text.DecimalFormat;

import static mpredict.Options.*;
import static mpredict.Options.*;

import mpredict.Predict.CutoffBasis;

public class Tester {
    public Tester() {
        super();
    }

    public static void main(String[] args) {
        String userdir          = System.getProperty("user.dir");
        String inFileMatrix     = userdir+"/input/PWMs.txt";
        String inFileCutoff     = null;//userdir+"/input/PWMs.cutoff.txt";
        String inFileSequences  = userdir+"/input/Sequences/gata1.FASTA";
        String inFileRandom     = userdir+"/input/randomseq1000k.1.txt";
        String outFileDat       = null;
        String outFileSig       = null;
        String outFileFim       = null;

        float pseudo        = 0.375f;
        long time           = System.currentTimeMillis();
        
        int p = 0,
            c = 0,
            s = 0,
            r = 0;
        float percentage = .9999f;
        int randSize = 100000;
        boolean formatLong = false;
        boolean useRandom = false;
        CutoffBasis basis = CutoffBasis.PWM;

        try{
          Predict pre = new Predict();
        
        /**Required Parameters**/
          p = pre.loadPWMs(inFileMatrix, pseudo);
          s = pre.loadSequences(inFileSequences, Sequence.SequenceType.STANDARD);
          c = pre.loadMotifCutOffs(inFileCutoff);           
          outFileDat = inFileSequences+".dat";
          outFileFim = inFileSequences+".fim";
          outFileSig = inFileSequences+".sig";
          
          useRandom = true;

          pre.setPercentCut(percentage);

        /** Brief Output displays relevant information **/
          DecimalFormat df = new DecimalFormat("00.00%");
          System.out.println("Read in "+p+" PWMS, "+c+" cutoffs, "+s+" Sequences.");
          System.out.println("Using sequence file "+inFileSequences+" and "+df.format(percentage)+" cutoff");          
          
          if(useRandom){
            SequenceRANDOM rand = new SequenceRANDOM(
                                        Sequence.loadSequences(inFileRandom, Sequence.SequenceType.RANDOM, 0), 
                                        randSize);
            pre.createCutoffs(rand, percentage, basis);
          }
          if(r > 0){
            System.out.println("Read in "+r+" Random Sequences");
          }

        /** Run the program **/
          pre.createTrx(outFileDat, formatLong);
          IFimProcess fimMax = new FimMAX(userdir+"/fim_maximal", 100, outFileDat, outFileFim);
          Poisson poisson = Poisson.getInstance();
          
          pre.printMotifFrequency();

          poisson.init(pre.getMotifFrequency(),
                       pre.getNonRandomSequenceLength(),
                       pre.getNonRandomNumSequences(),
                       outFileSig,
                       fimMax);

          poisson.CalculatePoisson();
        }
        catch(IOException e) {
          System.out.println("Error reading input or writing output file. Please check paths and names.");
          System.out.println(e.getMessage());
        }
        time = System.currentTimeMillis() - time;
        System.out.println("\nexecution time: "+time/(1000*60)+" min.");
    }
    
    
  /**
   * Returns the Usage string
   */
    public static String getUsage(){
      StringBuffer sb = new StringBuffer();
      sb.append("java MPredict [options] <PWM File in> <Sequences.Fasta in> \n")
        .append("-c <Cutoff file in> Cutoff file format\n")
        .append("-r <Random.Fasta in> .FASTA file format\n")
        .append("\t -b <% cutoff basis>: (p)wm or (s)equence>, default: (p)wm\n")
        .append("\t -per <[0-1]>: .60 = cutoff default \n")
        .append("-l Output results in long format\n")
        ;
      return sb.toString();
    }
  
  
  /**
   * Displays Message, Usage and Exits program 
   * @param message
   */
    public static void doUserExit(String message){
    System.out.println(message);
    System.out.println(getUsage());
    System.exit(1);
  }
  
}
